
#include <Servo.h>
#include <TM1637Display.h>

// --- PIN DEFINITIONS ---
const int trig = 11;
const int echo = 12;
const int ServoPin = 10;
const int buzzerPin = A2;
const int redPin = 13;
const int greenPin = A0;
const int bluePin = A1;
const int buttonPin = 9;
const int switchPin = A4;

const int CLK = A5;  // TM1637 clock pin
const int DIO = A3;  // TM1637 data pin

const int LEDS[] = {2, 3, 4, 5, 6, 7, 8};

Servo servo;
TM1637Display display(CLK, DIO);

// --- VARIABLES ---
int mode = 1;
bool lastButtonState = LOW;
bool currentButtonState = LOW;
int lastSwitchState = LOW;
int currentSwitchState = LOW;
int duration = 0;
int distance = 0;
bool systemEnabled = false;

void setup() {
  Serial.begin(9600);

  pinMode(trig, OUTPUT);
  pinMode(echo, INPUT);
  pinMode(buttonPin, INPUT);
  pinMode(switchPin, INPUT);
  pinMode(buzzerPin, OUTPUT);

  for (int i = 0; i < 7; i++) pinMode(LEDS[i], OUTPUT);
  pinMode(redPin, OUTPUT);
  pinMode(greenPin, OUTPUT);
  pinMode(bluePin, OUTPUT);

  servo.attach(ServoPin);
  display.setBrightness(5); // adjust brightness 0–7
  display.clear();
}

void loop() {
  // --- SLIDE SWITCH ---
  currentSwitchState = digitalRead(switchPin);
  if (currentSwitchState != lastSwitchState) {
    systemEnabled = currentSwitchState == HIGH;
    playToggleMusic();
    lastSwitchState = currentSwitchState;
    Serial.println(systemEnabled ? "System ON" : "System OFF");
  }

  if (!systemEnabled) {
    setRGB(0, 0, 0);
    for (int i = 0; i < 7; i++) digitalWrite(LEDS[i], LOW);
    noTone(buzzerPin);
    display.clear();
    servo.detach();
    return;
  } else {
    if (!servo.attached()) servo.attach(ServoPin);
  }

  // --- BUTTON MODE SWITCH ---
  currentButtonState = digitalRead(buttonPin);
  if (currentButtonState == HIGH && lastButtonState == LOW) {
    mode++;
    if (mode > 3) mode = 1;
    playBeepForMode(mode);
    Serial.print("Mode changed to: ");
    Serial.println(mode);
    delay(200);
  }
  lastButtonState = currentButtonState;

  // --- MODE RGB INDICATOR ---
  switch (mode) {
    case 1: setRGB(0, 255, 0); break;  // Green
    case 2: setRGB(255, 0, 0); break;  // Red
    case 3: setRGB(0, 0, 255); break;  // Blue
  }

  // --- MEASURE DISTANCE ---
  digitalWrite(trig, LOW); delayMicroseconds(2);
  digitalWrite(trig, HIGH); delayMicroseconds(10);
  digitalWrite(trig, LOW);
  duration = pulseIn(echo, HIGH);
  distance = (duration / 2) / 28.5;
  Serial.print("Distance: ");
  Serial.println(distance);

  // --- DISPLAY MODE + DISTANCE ---
  uint8_t data[4];
  data[0] = display.encodeDigit(mode);                      // First digit = mode
  data[1] = 0x40;                                           // Second digit = dash (-)
  int safeDistance = min(distance, 99);                     // Cap at 99
  data[2] = display.encodeDigit(safeDistance / 10);         // Tens
  data[3] = display.encodeDigit(safeDistance % 10);         // Ones
  display.setSegments(data);

  // --- LED BAR MODE 1 & 3 ---
  if (distance > 5) {
    bool ledActive = (mode == 1 || mode == 3);
    int dists[] = {25, 20, 17, 15, 10, 7, 5};
    for (int i = 0; i < 7; i++) {
      digitalWrite(LEDS[i], ledActive && distance <= dists[i]);
    }
  }

  // --- SERVO MODES 2 & 3 ---
  if ((mode == 2 || mode == 3) && distance <= 25) {
    int angle = map(distance, 25, 5, 0, 180);
    angle = constrain(angle, 0, 180);
    servo.write(angle);
  }

  // --- ALERT SYSTEM WHEN < 5cm ---
  if (distance <= 5 && (mode == 2 || mode == 3)) {
    for (int i = 0; i < 7; i++) {
      digitalWrite(LEDS[i], HIGH);
      tone(buzzerPin, 400);
      delay(100);
      digitalWrite(LEDS[i], LOW);
      noTone(buzzerPin);
      delay(100);
    }
  }

  delay(100);
}

// --- FUNCTIONS ---
void setRGB(int r, int g, int b) {
  analogWrite(redPin, r);
  analogWrite(greenPin, g);
  analogWrite(bluePin, b);
}

void playBeepForMode(int mode) {
  for (int i = 0; i < mode; i++) {
    tone(buzzerPin, 800); delay(150);
    noTone(buzzerPin); delay(100);
  }
}

void playToggleMusic() {
  tone(buzzerPin, 600); delay(160);
  noTone(buzzerPin); delay(100);
  tone(buzzerPin, 700); delay(160);
  noTone(buzzerPin); delay(100);
  tone(buzzerPin, 900); delay(220);
  noTone(buzzerPin);
}
